/****************************************************************************
 *
 * Copy right:   RL.Chen J and Zhen C
 * File name:    AppFunc.c
 * Description:  应用层函数调用文件
 * History:      2021-03-15
 *
 ****************************************************************************/
#include "AppFunc.h"
#include "AppSignal.h"
/*******
 * Fota升级处理函数
 * 输入：网络接收的数据，网络链接ID
 * 输出：0x00表示数据未准备完成，0x01表示数据模块OTA数据准备完成，0x88表示BMS的OTA数据准备完成
 * 处理：将接收的数据进行校验，搬运至升级区域，并进行应答
*******/
UINT8 Fota_Func(UINT8 *DataPtr,INT32 connectId)
{
    UINT8 		Fota_Answer[43]={0};
	Fota_Type 	Fota_S;
    UINT8 		Fota_Cmd = 0;
    INT8 		ret = 0;
    UINT8* 		Data_Read_Buffer = NULL;
    UINT8 		Data_Read_Crc = 0;
    if(*(DataPtr+30)==0x01)
    {
        Fota_S.Fota_Flash_Addres = FLASH_FOTA_REGION_START;
        Fota_Cmd = *(DataPtr+31);
        Fota_Answer[0] = TCP_START_SYM1;
        Fota_Answer[1] = TCP_START_SYM2;
        Fota_Answer[2] = TCP_CONCMD_SYM;
        switch (Fota_Cmd)
        {
            case 0x01:
            {
                Fota_S.Fota_All_Data_Len = *(DataPtr+33)<<24|*(DataPtr+34)<<16|*(DataPtr+35)<<8|*(DataPtr+36);
                Fota_S.Fota_Current_Addres = *(DataPtr+37)<<24|*(DataPtr+38)<<16|*(DataPtr+39)<<8|*(DataPtr+40);
                if(Fota_S.Fota_All_Data_Len>=(FLASH_BMS_FOTA_START_ADDR - FLASH_FOTA_REGION_START))
                {
                    Fota_Answer[3] = 0x02;
                }
                else
                {
                    Fota_Answer[3] = 0x01;
                    BSP_QSPI_Erase_Safe(Fota_S.Fota_Flash_Addres,Fota_S.Fota_All_Data_Len + 4 - (Fota_S.Fota_All_Data_Len%4)); //对齐擦除
                }
                memcpy(&Fota_Answer[4],(DataPtr+4),BATT_SN_LEN);
                Fota_Answer[21] = TCP_ENCPT_DISABLE;
                Fota_Answer[22] = 0x00;
                Fota_Answer[23] = 0x12;
                memcpy(&Fota_Answer[24],(DataPtr+24),18);
                Fota_Answer[42] =  bcc_chk_fota(Fota_Answer,42);
                tcpipConnectionSend(connectId,Fota_Answer,43,0,0,0);
                break;
            }
            case 0x02:
            {
                Fota_S.Fota_All_Data_Len = *(DataPtr+33)<<24|*(DataPtr+34)<<16|*(DataPtr+35)<<8|*(DataPtr+36);
                Fota_S.Fota_Current_Addres = *(DataPtr+37)<<24|*(DataPtr+38)<<16|*(DataPtr+39)<<8|*(DataPtr+40);
                Fota_S.Fota_Recv_Data_Len = *(DataPtr+41);
                memset(Fota_S.Fota_Recv_Data,0x00,100);
                memcpy(Fota_S.Fota_Recv_Data,(DataPtr+42),*(DataPtr+41));
                Fota_S.Fota_CRC = Fota_crc_chk(Fota_S.Fota_Recv_Data,Fota_S.Fota_Recv_Data_Len);
                if(Fota_S.Fota_CRC == *(DataPtr+Fota_S.Fota_Recv_Data_Len+42))
                {
                    if(Fota_S.Fota_Recv_Data_Len%4!=0)
                    {
                    Fota_S.Fota_Recv_Data_Len = Fota_S.Fota_Recv_Data_Len + 4-(Fota_S.Fota_Recv_Data_Len%4);
                    }
                    BSP_QSPI_Write_Safe(Fota_S.Fota_Recv_Data,Fota_S.Fota_Flash_Addres+Fota_S.Fota_Current_Addres,Fota_S.Fota_Recv_Data_Len);
                    Data_Read_Buffer = malloc(Fota_S.Fota_Recv_Data_Len);
                    BSP_QSPI_Read_Safe(Data_Read_Buffer,Fota_S.Fota_Flash_Addres+Fota_S.Fota_Current_Addres,Fota_S.Fota_Recv_Data_Len);
                    Data_Read_Crc = Fota_crc_chk(Data_Read_Buffer,Fota_S.Fota_Recv_Data_Len);
                    #ifdef USING_PRINTF
                        printf("\n\n\n");
                        for(int i=0;i<Fota_S.Fota_Recv_Data_Len;i++)
                        {
                            printf("%x  ",*(Data_Read_Buffer+i));
                        }    
                        printf("\n\n\n");
                    #endif
					if(Data_Read_Buffer!=NULL)
                    	free(Data_Read_Buffer);
					Data_Read_Buffer = NULL;
                    if(Data_Read_Crc==Fota_S.Fota_CRC )
                    {
                        Fota_Answer[3] = 0x01;
                    }
                    else
                    {
                        Fota_Answer[3] = 0x02;
                        BSP_QSPI_Erase_Safe(Fota_S.Fota_Flash_Addres+Fota_S.Fota_Current_Addres,Fota_S.Fota_Recv_Data_Len);
                    }
                }
                else//数据校验失败
                {
                    Fota_Answer[3] = 0x02;
                }
                memcpy(&Fota_Answer[4],(DataPtr+4),BATT_SN_LEN);
                Fota_Answer[21] = TCP_ENCPT_DISABLE;
                Fota_Answer[22] = 0x00;
                Fota_Answer[23] = 0x12;
                memcpy(&Fota_Answer[24],(DataPtr+24),18);
                Fota_Answer[42] =  bcc_chk_fota(Fota_Answer,42);
                tcpipConnectionSend(connectId,Fota_Answer,43,0,0,0);
                break;
            }
            case 0x03:
            {
                Fota_S.Fota_All_Data_Len = *(DataPtr+33)<<24|*(DataPtr+34)<<16|*(DataPtr+35)<<8|*(DataPtr+36);
                Fota_S.Fota_Current_Addres = *(DataPtr+37)<<24|*(DataPtr+38)<<16|*(DataPtr+39)<<8|*(DataPtr+40);
                Fota_Answer[3] = 0x01;
                memcpy(&Fota_Answer[4],(DataPtr+4),BATT_SN_LEN);
                Fota_Answer[21] = TCP_ENCPT_DISABLE;
                Fota_Answer[22] = 0x00;
                Fota_Answer[23] = 0x12;
                memcpy(&Fota_Answer[24],(DataPtr+24),18);
                Fota_Answer[42] =  bcc_chk_fota(Fota_Answer,42);
                tcpipConnectionSend(connectId,Fota_Answer,43,0,0,0);
                if(Fota_S.Fota_All_Data_Len==Fota_S.Fota_Current_Addres)
                {
                    return 0x01;
                }
                else
                {
                    return 0x00;
                }
                break;
            }
            default:
            {
                Fota_Answer[3] = 0x02;
                memcpy(&Fota_Answer[4],(DataPtr+4),BATT_SN_LEN);
                Fota_Answer[21] = TCP_ENCPT_DISABLE;
                Fota_Answer[22] = 0x00;
                Fota_Answer[23] = 0x12;
                memcpy(&Fota_Answer[24],(DataPtr+24),18);
                Fota_Answer[42] =  bcc_chk_fota(Fota_Answer,42);
                tcpipConnectionSend(connectId,Fota_Answer,43,0,0,0);
                break;
            }
                
        }
		return 0x00;
    }
    else if(*(DataPtr+30)==0x88)//BMS升级文件存放
    {
        Fota_S.Fota_Flash_Addres = FLASH_BMS_FOTA_START_ADDR;
        Fota_Cmd = *(DataPtr+31);
        Fota_Answer[0] = TCP_START_SYM1;
        Fota_Answer[1] = TCP_START_SYM2;
        Fota_Answer[2] = TCP_CONCMD_SYM;
        switch (Fota_Cmd)
        {
            case 0x01:
            {
                Fota_S.Fota_All_Data_Len = *(DataPtr+33)<<24|*(DataPtr+34)<<16|*(DataPtr+35)<<8|*(DataPtr+36);
                Fota_S.Fota_Current_Addres = *(DataPtr+37)<<24|*(DataPtr+38)<<16|*(DataPtr+39)<<8|*(DataPtr+40);
                if(Fota_S.Fota_All_Data_Len>=(FLASH_BMS_FOTA_END_ADDR - FLASH_BMS_FOTA_START_ADDR))
                {
                    Fota_Answer[3] = 0x02;
                }
                else
                {
                    Fota_Answer[3] = 0x01;
                }
                memcpy(&Fota_Answer[4],(DataPtr+4),BATT_SN_LEN);
                Fota_Answer[21] = TCP_ENCPT_DISABLE;
                Fota_Answer[22] = 0x00;
                Fota_Answer[23] = 0x12;
                memcpy(&Fota_Answer[24],(DataPtr+24),18);
                Fota_Answer[42] =  bcc_chk_fota(Fota_Answer,42);
                tcpipConnectionSend(connectId,Fota_Answer,43,0,0,0);
                if(Fota_Answer[3] == 0x01)
                {
                    BSP_QSPI_Erase_Safe(Fota_S.Fota_Flash_Addres,Fota_S.Fota_All_Data_Len + 4 - (Fota_S.Fota_All_Data_Len%4)); //512k-32k = 480k -> 0x75300  0x78000
                }
                break;
            }
            case 0x02:
            {
                Fota_S.Fota_All_Data_Len = *(DataPtr+33)<<24|*(DataPtr+34)<<16|*(DataPtr+35)<<8|*(DataPtr+36);
                Fota_S.Fota_Current_Addres = *(DataPtr+37)<<24|*(DataPtr+38)<<16|*(DataPtr+39)<<8|*(DataPtr+40);
                Fota_S.Fota_Recv_Data_Len = *(DataPtr+41);
                memset(Fota_S.Fota_Recv_Data,0x00,100);
                memcpy(Fota_S.Fota_Recv_Data,(DataPtr+42),*(DataPtr+41));
                Fota_S.Fota_CRC = Fota_crc_chk(Fota_S.Fota_Recv_Data,Fota_S.Fota_Recv_Data_Len);
                if(Fota_S.Fota_CRC == *(DataPtr+Fota_S.Fota_Recv_Data_Len+42))
                {
                    if(Fota_S.Fota_Recv_Data_Len%4!=0)
                    {
                    Fota_S.Fota_Recv_Data_Len = Fota_S.Fota_Recv_Data_Len + 4-(Fota_S.Fota_Recv_Data_Len%4);
                    }
                    BSP_QSPI_Write_Safe(Fota_S.Fota_Recv_Data,Fota_S.Fota_Flash_Addres+Fota_S.Fota_Current_Addres,Fota_S.Fota_Recv_Data_Len);
                    Data_Read_Buffer = malloc(Fota_S.Fota_Recv_Data_Len);
                    BSP_QSPI_Read_Safe(Data_Read_Buffer,Fota_S.Fota_Flash_Addres+Fota_S.Fota_Current_Addres,Fota_S.Fota_Recv_Data_Len);
                    Data_Read_Crc = Fota_crc_chk(Data_Read_Buffer,Fota_S.Fota_Recv_Data_Len);
                    #ifdef USING_PRINTF1
                        printf("\n\n\n");
                        UINT8 temp[1];
                        for(int i=0;i<Fota_S.Fota_Recv_Data_Len;i++)
                        {
                            printf("%x  ",*(Data_Read_Buffer+i));
                        }    
                        printf("\n\n\n");
                    #endif
                    if(Data_Read_Buffer!=NULL)
                    	free(Data_Read_Buffer);
					Data_Read_Buffer = NULL;
                    if(Data_Read_Crc==Fota_S.Fota_CRC )
                    {
                        Fota_Answer[3] = 0x01;
                    }
                    else
                    {
                        Fota_Answer[3] = 0x02;
                        BSP_QSPI_Erase_Safe(Fota_S.Fota_Flash_Addres+Fota_S.Fota_Current_Addres,Fota_S.Fota_Recv_Data_Len);
                    }
                }
                else//数据校验失败
                {
                    Fota_Answer[3] = 0x02;
                }
                memcpy(&Fota_Answer[4],(DataPtr+4),BATT_SN_LEN);
                Fota_Answer[21] = TCP_ENCPT_DISABLE;
                Fota_Answer[22] = 0x00;
                Fota_Answer[23] = 0x12;
                memcpy(&Fota_Answer[24],(DataPtr+24),18);
                Fota_Answer[42] =  bcc_chk_fota(Fota_Answer,42);
                tcpipConnectionSend(connectId,Fota_Answer,43,0,0,0);
                break;
            }
            case 0x03:
            {
                Fota_S.Fota_All_Data_Len = *(DataPtr+33)<<24|*(DataPtr+34)<<16|*(DataPtr+35)<<8|*(DataPtr+36);
                Fota_S.Fota_Current_Addres = *(DataPtr+37)<<24|*(DataPtr+38)<<16|*(DataPtr+39)<<8|*(DataPtr+40);
                Fota_Answer[3] = 0x01;
                memcpy(&Fota_Answer[4],(DataPtr+4),BATT_SN_LEN);
                Fota_Answer[21] = TCP_ENCPT_DISABLE;
                Fota_Answer[22] = 0x00;
                Fota_Answer[23] = 0x12;
                memcpy(&Fota_Answer[24],(DataPtr+24),18);
                Fota_Answer[42] =  bcc_chk_fota(Fota_Answer,42);
                tcpipConnectionSend(connectId,Fota_Answer,43,0,0,0);
                if(Fota_S.Fota_All_Data_Len==Fota_S.Fota_Current_Addres)
                {
                    return 0x88;
                }
                else
                {
                    return 0x00;
                }
                break;
            }
            default:
            {
                Fota_Answer[3] = 0x02;
                memcpy(&Fota_Answer[4],(DataPtr+4),BATT_SN_LEN);
                Fota_Answer[21] = TCP_ENCPT_DISABLE;
                Fota_Answer[22] = 0x00;
                Fota_Answer[23] = 0x12;
                memcpy(&Fota_Answer[24],(DataPtr+24),18);
                Fota_Answer[42] =  bcc_chk_fota(Fota_Answer,42);
                tcpipConnectionSend(connectId,Fota_Answer,43,0,0,0);
                break;
            }
        }
		return 0x00;
    }
}
static UINT8 bcc_chk_fota(UINT8* data, UINT8 length)
{
    UINT8 bcc_chk_return = 0x00;
    UINT8 count = 0;
    while (count<length)
    {
        bcc_chk_return^=data[count];
        count++;
    }
    return  bcc_chk_return;
}
static UINT8 Fota_crc_chk(UINT8* data,UINT8 length)
{  
    UINT8 reg_crc=0x00;
    while(length--)
    { 
        reg_crc ^= *data++;
    }
    return reg_crc;
}
/*******
 * Can解码组包函数
 * 输入：全局变量和缓存地址
 * 输出：空
 * 处理：将全局变量进行组包后存放
*******/
void CANEncodeFunction(UINT32 ID,UINT8* rawData)
{
	memset(rawData, 0xFF, 8);	
	switch (ID)
	{
		case	0x6A0:
			*(UINT16*)(rawData +0) = ((battCellU[0]>>8) & 0xFF) | (((battCellU[0]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +2) = ((battCellU[1]>>8) & 0xFF) | (((battCellU[1]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +4) = ((battCellU[2]>>8) & 0xFF) | (((battCellU[2]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +6) = ((battCellU[3]>>8) & 0xFF) | (((battCellU[3]&0xFF)<<8)&0xFF00);
			break;
		case	0x6A1:
			*(UINT16*)(rawData +0) = ((battCellU[4]>>8) & 0xFF) | (((battCellU[4]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +2) = ((battCellU[5]>>8) & 0xFF) | (((battCellU[5]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +4) = ((battCellU[6]>>8) & 0xFF) | (((battCellU[6]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +6) = ((battCellU[7]>>8) & 0xFF) | (((battCellU[7]&0xFF)<<8)&0xFF00);
			break;
		case	0x6A2:
			*(UINT16*)(rawData +0) = ((battCellU[8]>>8) & 0xFF) | (((battCellU[8]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +2) = ((battCellU[9]>>8) & 0xFF) | (((battCellU[9]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +4) = ((battCellU[10]>>8) & 0xFF) | (((battCellU[10]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +6) = ((battCellU[11]>>8) & 0xFF) | (((battCellU[11]&0xFF)<<8)&0xFF00);
			break;
		case	0x6A3:
			*(UINT16*)(rawData +0) = ((battCellU[12]>>8) & 0xFF) | (((battCellU[12]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +2) = ((battCellU[13]>>8) & 0xFF) | (((battCellU[13]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +4) = ((battCellU[14]>>8) & 0xFF) | (((battCellU[14]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +6) = ((battCellU[15]>>8) & 0xFF) | (((battCellU[15]&0xFF)<<8)&0xFF00);
			break;
		case	0x6A4:
			*(UINT16*)(rawData +0) = ((battCellU[16]>>8) & 0xFF) | (((battCellU[16]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +2) = ((battCellU[17]>>8) & 0xFF) | (((battCellU[17]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +4) = ((battCellU[18]>>8) & 0xFF) | (((battCellU[18]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +6) = ((battCellU[19]>>8) & 0xFF) | (((battCellU[19]&0xFF)<<8)&0xFF00);
			break;
		case	0x6A5:
			*(UINT16*)(rawData +0) = ((battCellU[20]>>8) & 0xFF) | (((battCellU[20]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +2) = ((battCellU[21]>>8) & 0xFF) | (((battCellU[21]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +4) = ((battCellU[22]>>8) & 0xFF) | (((battCellU[22]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +6) = ((battCellU[23]>>8) & 0xFF) | (((battCellU[23]&0xFF)<<8)&0xFF00);
			break;
		case	0x6A6:
			*(UINT16*)(rawData +0) = ((battCellU[24]>>8) & 0xFF) | (((battCellU[24]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +2) = ((battCellU[25]>>8) & 0xFF) | (((battCellU[25]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +4) = ((battCellU[26]>>8) & 0xFF) | (((battCellU[26]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +6) = ((battCellU[27]>>8) & 0xFF) | (((battCellU[27]&0xFF)<<8)&0xFF00);
			break;
		case	0x6B0:
			*(UINT8*)(rawData +0) =battCellTemp[0]&0xFF;
			*(UINT8*)(rawData +1) =battCellTemp[1]&0xFF;
			*(UINT8*)(rawData +2) =battCellTemp[2]&0xFF;
			*(UINT8*)(rawData +3) =battCellTemp[3]&0xFF;
			*(UINT8*)(rawData +4) =battCellTemp[4]&0xFF;
			*(UINT8*)(rawData +5) =battCellTemp[5]&0xFF;
			*(UINT8*)(rawData +6) =battCellTemp[6]&0xFF;
			*(UINT8*)(rawData +7) =battCellTemp[7]&0xFF;
			break;
		case	0x6BA:
			*(UINT8*)(rawData +0) =MOSTemp&0xFF;
			*(UINT8*)(rawData +1) =packTemp&0xFF;
			*(UINT8*)(rawData +2) =fastChargeTemp&0xFF;
			*(UINT8*)(rawData +3) =normalChargeTemp&0xFF;
			*(UINT8*)(rawData +4) =heatTemp1&0xFF;
			*(UINT8*)(rawData +5) =heatTemp2&0xFF;
			*(UINT8*)(rawData +6) =nbReservedTemp1&0xFF;
			*(UINT8*)(rawData +7) =nbReservedTemp2&0xFF;
			break;
		case	0x6C0:
			*(UINT16*)(rawData +0) = ((minCellVol>>8) & 0xFF) | (((minCellVol&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +2) = ((maxCellVol>>8) & 0xFF) | (((maxCellVol&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +4) = ((battPackVol>>8) & 0xFF) | (((battPackVol&0xFF)<<8)&0xFF00);
			*(UINT8*)(rawData +6) =maxCellTemp&0xFF;
			*(UINT8*)(rawData +7) =minCellTemp&0xFF;
			break;
		case	0x6C1:
			*(UINT8*)(rawData +0) =battSOC&0xFF;
			*(UINT8*)(rawData +1) =battSOH&0xFF;
			*(UINT16*)(rawData +2) = ((battI>>8) & 0xFF) | (((battI&0xFF)<<8)&0xFF00);
			*(UINT8*)(rawData +4) =battWorkState&0xFF;
			*(UINT32*)(rawData +5) = ((battWarningState>>16) & 0xFF)|((battWarningState)&0xFF00)|(((battWarningState&0xFF)<<16)&0xFF0000);
			break;
		case	0x6C2:
			*(UINT32*)(rawData +0) = ((nbSwVersion>>24) & 0xFF)|((nbSwVersion>>8)&0xFF00)|((nbSwVersion<<8)&0xFF0000)|(((nbSwVersion&0xFF)<<24)&0xFF000000);
			*(UINT16*)(rawData +4) = ((nbHwVersion>>8) & 0xFF) | (((nbHwVersion&0xFF)<<8)&0xFF00);
			*(UINT8*)(rawData +6) =bmsSwVersion&0xFF;
			*(UINT8*)(rawData +7) =bmsHwVersion&0xFF;
			break;
		case	0x6C3:
			*(UINT32*)(rawData +0) = ((battBalanceoInfo>>24) & 0xFF)|((battBalanceoInfo>>8)&0xFF00)|((battBalanceoInfo<<8)&0xFF0000)|(((battBalanceoInfo&0xFF)<<24)&0xFF000000);
			*(UINT8*)(rawData +4) =battMOSSwitchState&0xFF;
			*(UINT8*)(rawData +5) =battHeatEnableState&0xFF;
			break;
		case	0x6C4:
			*(UINT8*)(rawData +0) =reservedSignal1&0xFF;
			*(UINT8*)(rawData +1) =reservedSignal2&0xFF;
			*(UINT8*)(rawData +2) =reservedSignal3&0xFF;
			*(UINT8*)(rawData +3) =reservedSignal4&0xFF;
			*(UINT8*)(rawData +4) =reservedSignal5&0xFF;
			*(UINT8*)(rawData +5) =reservedSignal6&0xFF;
			*(UINT8*)(rawData +6) =reservedSignal7&0xFF;
			*(UINT8*)(rawData +7) =reservedSignal8&0xFF;
			break;
		case	0x7C0:
			*(UINT16*)(rawData +0) = ((battCellU[0]>>8) & 0xFF) | (((battCellU[0]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +2) = ((battCellU[1]>>8) & 0xFF) | (((battCellU[1]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +4) = ((battCellU[2]>>8) & 0xFF) | (((battCellU[2]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +6) = ((battCellU[3]>>8) & 0xFF) | (((battCellU[3]&0xFF)<<8)&0xFF00);
			break;
		case	0x7C1:
			*(UINT16*)(rawData +0) = ((battCellU[4]>>8) & 0xFF) | (((battCellU[4]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +2) = ((battCellU[5]>>8) & 0xFF) | (((battCellU[5]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +4) = ((battCellU[6]>>8) & 0xFF) | (((battCellU[6]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +6) = ((battCellU[7]>>8) & 0xFF) | (((battCellU[7]&0xFF)<<8)&0xFF00);
			break;
		case	0x7C2:
			*(UINT16*)(rawData +0) = ((battCellU[8]>>8) & 0xFF) | (((battCellU[8]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +2) = ((battCellU[9]>>8) & 0xFF) | (((battCellU[9]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +4) = ((battCellU[10]>>8) & 0xFF) | (((battCellU[10]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +6) = ((battCellU[11]>>8) & 0xFF) | (((battCellU[11]&0xFF)<<8)&0xFF00);
			break;
		case	0x7C3:
			*(UINT16*)(rawData +0) = ((battCellU[12]>>8) & 0xFF) | (((battCellU[12]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +2) = ((battCellU[13]>>8) & 0xFF) | (((battCellU[13]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +4) = ((battCellU[14]>>8) & 0xFF) | (((battCellU[14]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +6) = ((battCellU[15]>>8) & 0xFF) | (((battCellU[15]&0xFF)<<8)&0xFF00);
			break;
		case	0x7C4:
			*(UINT16*)(rawData +0) = ((battCellU[16]>>8) & 0xFF) | (((battCellU[16]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +2) = ((battCellU[17]>>8) & 0xFF) | (((battCellU[17]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +4) = ((battCellU[18]>>8) & 0xFF) | (((battCellU[18]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +6) = ((battCellU[19]>>8) & 0xFF) | (((battCellU[19]&0xFF)<<8)&0xFF00);
			break;
		case	0x7C5:
			*(UINT16*)(rawData +0) = ((battCellU[20]>>8) & 0xFF) | (((battCellU[20]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +2) = ((battCellU[21]>>8) & 0xFF) | (((battCellU[21]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +4) = ((battCellU[22]>>8) & 0xFF) | (((battCellU[22]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +6) = ((battCellU[23]>>8) & 0xFF) | (((battCellU[23]&0xFF)<<8)&0xFF00);
			break;
		case	0x7C6:
			*(UINT16*)(rawData +0) = ((battCellU[24]>>8) & 0xFF) | (((battCellU[24]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +2) = ((battCellU[25]>>8) & 0xFF) | (((battCellU[25]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +4) = ((battCellU[26]>>8) & 0xFF) | (((battCellU[26]&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +6) = ((battCellU[27]>>8) & 0xFF) | (((battCellU[27]&0xFF)<<8)&0xFF00);
			break;
		case	0x7C7:
			*(UINT8*)(rawData +0) =battCellTemp[0]&0xFF;
			*(UINT8*)(rawData +1) =battCellTemp[1]&0xFF;
			*(UINT8*)(rawData +2) =battCellTemp[2]&0xFF;
			*(UINT8*)(rawData +3) =battCellTemp[3]&0xFF;
			*(UINT8*)(rawData +4) =battCellTemp[4]&0xFF;
			*(UINT8*)(rawData +5) =battCellTemp[5]&0xFF;
			*(UINT8*)(rawData +6) =battCellTemp[6]&0xFF;
			*(UINT8*)(rawData +7) =battCellTemp[7]&0xFF;
			break;
		case	0x7C8:
			*(UINT8*)(rawData +0) =MOSTemp&0xFF;
			*(UINT8*)(rawData +1) =packTemp&0xFF;
			*(UINT8*)(rawData +2) =fastChargeTemp&0xFF;
			*(UINT8*)(rawData +3) =normalChargeTemp&0xFF;
			*(UINT8*)(rawData +4) =heatTemp1&0xFF;
			*(UINT8*)(rawData +5) =heatTemp2&0xFF;
			*(UINT8*)(rawData +6) =nbReservedTemp1&0xFF;
			*(UINT8*)(rawData +7) =nbReservedTemp2&0xFF;
			break;
		case	0x7C9:
			*(UINT16*)(rawData +0) = ((minCellVol>>8) & 0xFF) | (((minCellVol&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +2) = ((maxCellVol>>8) & 0xFF) | (((maxCellVol&0xFF)<<8)&0xFF00);
			*(UINT16*)(rawData +4) = ((battPackVol>>8) & 0xFF) | (((battPackVol&0xFF)<<8)&0xFF00);
			*(UINT8*)(rawData +6) =maxCellTemp&0xFF;
			*(UINT8*)(rawData +7) =minCellTemp&0xFF;
			break;
		case	0x7CA:
			*(UINT8*)(rawData +0) =battSOC&0xFF;
			*(UINT8*)(rawData +1) =battSOH&0xFF;
			*(UINT16*)(rawData +2) = ((battI>>8) & 0xFF) | (((battI&0xFF)<<8)&0xFF00);
			*(UINT8*)(rawData +4) =battWorkState&0xFF;
			*(UINT32*)(rawData +5) = ((battWarningState>>16) & 0xFF)|((battWarningState)&0xFF00)|(((battWarningState&0xFF)<<16)&0xFF0000);
			break;
		case	0x7CB:
			*(UINT32*)(rawData +0) = ((nbSwVersion>>24) & 0xFF)|((nbSwVersion>>8)&0xFF00)|((nbSwVersion<<8)&0xFF0000)|(((nbSwVersion&0xFF)<<24)&0xFF000000);
			*(UINT16*)(rawData +4) = ((nbHwVersion>>8) & 0xFF) | (((nbHwVersion&0xFF)<<8)&0xFF00);
			*(UINT8*)(rawData +6) =bmsSwVersion&0xFF;
			*(UINT8*)(rawData +7) =bmsHwVersion&0xFF;
			break;
		case	0x7CC:
			*(UINT32*)(rawData +0) = ((battBalanceoInfo>>24) & 0xFF)|((battBalanceoInfo>>8)&0xFF00)|((battBalanceoInfo<<8)&0xFF0000)|(((battBalanceoInfo&0xFF)<<24)&0xFF000000);
			*(UINT8*)(rawData +4) =battMOSSwitchState&0xFF;
			*(UINT8*)(rawData +5) =battHeatEnableState&0xFF;
			break;
		case	0x7CD:
			*(UINT8*)(rawData +0) =reservedSignal1&0xFF;
			*(UINT8*)(rawData +1) =reservedSignal2&0xFF;
			*(UINT8*)(rawData +2) =reservedSignal3&0xFF;
			*(UINT8*)(rawData +3) =reservedSignal4&0xFF;
			*(UINT8*)(rawData +4) =reservedSignal5&0xFF;
			*(UINT8*)(rawData +5) =reservedSignal6&0xFF;
			*(UINT8*)(rawData +6) =reservedSignal7&0xFF;
			*(UINT8*)(rawData +7) =reservedSignal8&0xFF;
			break;
		case	0x7B0:
			*(UINT8*)(rawData +0) = (AppNVMData.battSN[0])&0xFF;
			*(UINT8*)(rawData +1) = (AppNVMData.battSN[1])&0xFF;
			*(UINT8*)(rawData +2) = (AppNVMData.battSN[2])&0xFF;
			*(UINT8*)(rawData +3) = (AppNVMData.battSN[3])&0xFF;
			*(UINT8*)(rawData +4) = (AppNVMData.battSN[4])&0xFF;
			*(UINT8*)(rawData +5) = (AppNVMData.battSN[5])&0xFF;
			*(UINT8*)(rawData +6) = (AppNVMData.battSN[6])&0xFF;
			*(UINT8*)(rawData +7) = (AppNVMData.battSN[7])&0xFF;
			break;
		case	0x7B1:
			*(UINT8*)(rawData +0) = (AppNVMData.battSN[8])&0xFF;
			*(UINT8*)(rawData +1) = (AppNVMData.battSN[9])&0xFF;
			*(UINT8*)(rawData +2) = (AppNVMData.battSN[10])&0xFF;
			*(UINT8*)(rawData +3) = (AppNVMData.battSN[11])&0xFF;
			*(UINT8*)(rawData +4) = (AppNVMData.battSN[12])&0xFF;
			*(UINT8*)(rawData +5) = (AppNVMData.battSN[13])&0xFF;
			*(UINT8*)(rawData +6) = (AppNVMData.battSN[14])&0xFF;
			*(UINT8*)(rawData +7) = (AppNVMData.battSN[15])&0xFF;
			break;
		case	0x7B2:
			*(UINT8*)(rawData +0) = (AppNVMData.battSN[16])&0xFF;
			
			break;
		
		default:
			break;
	}
}
/*******
 * 下线检测诊断函数
 * 输入：空
 * 输出：空
 * 处理：下线检测诊断
*******/

void UDS_Service()
{
	static UINT8	UDSService[2];
	static UINT8	UDSSubService[2];
	static UINT8	UDSSubServiceActionCode[2];
	static UINT8 	UDSSwitch = 0;
	static UINT8   UDSDialogMode = 01;
	static UINT32  updateDifferDataByteLen = 0;
	//each package means 100 bytes, max PackageCounter = updateDifferDataByteLen%100 + 1
	static UINT16  updateDifferDataPackageCounter = 0;  
	//each slice means 4 bytes, max SliceCounter = 25;
	static UINT32  updateDifferDataSliceCounter = 0;
	static UINT8   UDSBattSN[BATT_SN_LEN];
	static BOOL downloadReady = FALSE;
	UINT8 i,j= 0;
	UINT32 k=0;
	INT8 ret,errorCount = 0;
	BOOL boolRet = false;
	static UINT8 snFlag = 0;
	static UINT32 sliceCounterFlag = 0;
	UINT32 tempSliceCounter=0;	
	static UINT8 counter=0;  
	CAN_Msg_Type UDSAnswer ={0};
	UINT8   updateDataBuffer[100];
	UINT32 	posCode,negCode;
	
	
	//UINT8 temp00[94] ={0x00,0x0E,0x00,0x05,0x0D,0xDE,0x0D,0xDE,0x0D,0xDE,0x0D,0xDE,0x0D,0x7A,0x0D,0xDE,0x0D,0xDE,0x0D,0xDE,0x0D,0xDE,0x0D,0xDE,0x0E,0x42,0x0D,0xDE,0x0D,0xDE,0x0D,0xDE,0x13,0x38,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x21,0x00,0x24,0x00,0x26,0x00,0x22,0x00,0x24,0x00,0x00,0x00,0x00,0x6e,0x78,0x00,0x00,0x00,0x00,0x00,0x5F,0x00,0x62,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x13,0x74,0x0E,0x42,0x0D,0x7A,0x00,0x00};
	//MEMCPY(battInfo, temp00, 94);
	
	//while(true)
	//{
		//if(UDSSwitch == 1)
		{
			
			
			for(i=0; i<2; i++)
			{
				UDSAnswer.DLC = 8;
				memset(UDSAnswer.Data, 0, UDSAnswer.DLC);
				
				if(CanRxMsg[i].Id == 0x7A0)
					{
					switch (UDSService[i])
						{
							case 0x10:
								if(UDSSubService[i] == 0x01)
								{
									if((UDSDialogMode == 0x01 || UDSDialogMode==0x02)||UDSSwitch == 0)
									{									
										UDSPositiveAnswer(0x04,i,0x00);
										UDSSwitch = 0;
										UDSDialogMode = 1;
									}
									else
									{
										UDSNegtiveAnswer(0x05,i,0xFF);
									}
								}
								else if(UDSSubService[i] == 0x02)
								{
									UDSPositiveAnswer(0x04,i,0x00);
									UDSSwitch = 1;
									UDSDialogMode = 2;									
								}
								else if(UDSSubService[i] == 0x03)
								{
									if(UDSSwitch == 1)
									{
										UDSPositiveAnswer(0x04,i,0x00);
										UDSSwitch = 1;
										UDSDialogMode = 3;
			                		}
			                		else
			                		{
										UDSNegtiveAnswer(0x05,i,0xFF);
			                		}

								}
								else if(UDSSubService[i] == 0x11)   //make NB software reset
								{
									if(UDSDialogMode == 2)
									{
										UDSPositiveAnswer(0x04,i,0x00);										
										osDelay(100);
										appSaveNVMData();
										appSetCFUN(0);
										osDelay(1000);
			                			EC_SystemReset();
			                		}
			                		else
			                		{
										UDSNegtiveAnswer(0x05,i,0xFF);  
			                		}
								}								
								else
								{
									UDSNegtiveAnswer(0x05,i,0xEE);
								}
								break;
							case 0x22:
								if(UDSDialogMode == 2)
								{
									if(UDSSubService[i] == 0x01)		//check the sw of NB
									{
										UDSPositiveAnswer(0x08,i,APPSWVERSION);
									}
									else if(UDSSubService[i] == 0x02)  //check the hw of NB
									{
										UDSPositiveAnswer(0x06,i,HWVERSION);										
									}
									else if(UDSSubService[i] == 0x03)  //check the SN number
									{											
										switch(UDSSubServiceActionCode[i])
										{
											case 00:
												UDSAnswer.Id = 0x7B0;
												
												CANEncodeFunction(UDSAnswer.Id, UDSAnswer.Data);
												ret = HAL_Can_Transmit(UDSAnswer);
												
												break;
												
											case 01:
												UDSAnswer.Id = 0x7B1;
												CANEncodeFunction(UDSAnswer.Id, UDSAnswer.Data);
												ret = HAL_Can_Transmit(UDSAnswer);

												break;

											case 02:
												UDSAnswer.Id = 0x7B2;
												CANEncodeFunction(UDSAnswer.Id, UDSAnswer.Data);
												ret = HAL_Can_Transmit(UDSAnswer);
												
												break;
											default:
											break;											
										}									
									}
									else if(UDSSubService[i] == 0x04)  //check the batt message
									{
										UDSAnswer.Id = 0x7C0+UDSSubServiceActionCode[i];
										CANEncodeFunction(UDSAnswer.Id, UDSAnswer.Data);										
										ret = HAL_Can_Transmit(UDSAnswer);										
									}
									else if(UDSSubService[i] == 0x05)  //check the enviroment temp of NB
									{				
										UINT32 temp = ((fastChargeTemp<<24)&0xFF000000) | ((normalChargeTemp<<16)&0xFF0000) | ((heatTemp1<<8)&0xFF00) | (heatTemp2&0xFF);				
										UDSPositiveAnswer(0x08,i,temp);  //ntcvalue										
									}									
									else if(UDSSubService[i] == 0x06)	//check the tcp link of NB
									{
										UDSPositiveAnswer(0x06,i,TcpSendLen);  //TcpconnectStatus
									}
									else if(UDSSubService[i] == 0x07)  //check the GPS link of NB
									{
										posCode =(UINT32) PosFlag;
										UDSPositiveAnswer(0x05,i,posCode);  //gps satellite num(uint8), should be modified
									}	
									else
									{
										UDSNegtiveAnswer(0x05,i,0xEE);
									}
								}
								else	//the service is not surpported in current dialog mode
								{
									UDSNegtiveAnswer(0x05,i,0xFF);
								}
								break;
							case 0x2E:   //write service
								if(UDSDialogMode == 2)
								{
									if(UDSSubService[i] == 0x03)  // write the battSN
									{
										if(UDSSubServiceActionCode[i] == 0x00 && snFlag == 0x00)
										{
											for(j=0; j<5; j++)
											{
												UDSBattSN[j+5*0] = CanRxMsg[i].Data[j+3];
											}	
											snFlag = snFlag|0x01;
										}
										else if(UDSSubServiceActionCode[i] == 0x01 && snFlag == 0x01)
										{
											for(j=0; j<5; j++)
											{
												UDSBattSN[j+5*1] = CanRxMsg[i].Data[j+3];
											}
											snFlag = snFlag|0x02;
										}
										else if(UDSSubServiceActionCode[i] == 0x02 && snFlag == 0x03)
										{
											for(j=0; j<5; j++)
											{
												UDSBattSN[j+5*2] = CanRxMsg[i].Data[j+3];
											}
											snFlag = snFlag|0x04;
										}
										else if(UDSSubServiceActionCode[i] == 0x03 && snFlag == 0x07)
										{
											for(j=0; j<2; j++)
											{
												UDSBattSN[j+5*3] = CanRxMsg[i].Data[j+3];
											}
											snFlag = snFlag|0x08;
										}

										if(snFlag == 0x0F)
										{
											snFlag = 0;
											UDSPositiveAnswer(0x04,i,00);
											MEMCPY(AppConfigData.battSN, UDSBattSN, BATT_SN_LEN);
											AppConfigData.EOLState = 1; //SN号写入完成，表明已经进行过下线配置
											appSaveNVMData();
										}
										else
										{
											UDSAnswer.Id = 0x7A8;											
											UDSAnswer.Data[0] = 0x05;
											UDSAnswer.Data[1] = 0x3E;
											UDSAnswer.Data[2] = UDSService[i];
											UDSAnswer.Data[3] = UDSSubService[i];
											UDSAnswer.Data[4] = snFlag;	
											
											ret = HAL_Can_Transmit(UDSAnswer);
										}										
									}
									else if(UDSSubService[i] == 0x0F) //write the update config:updateDifferDataByteLen
									{										
										updateDifferDataByteLen = (CanRxMsg[i].Data[2]<<16)|(CanRxMsg[i].Data[3]<<8)|CanRxMsg[i].Data[4];										
										UDSPositiveAnswer(0x04,i,00);
									}
									else
									{
										UDSNegtiveAnswer(0x05,i,0xEE);
										
									}
									break;
								}								
								else
								{
									UDSNegtiveAnswer(0x05,i,0xFF);
								}
								break;
							case 0x31:  //clear the flash service
								if(UDSDialogMode == 3)  
								{
									if(UDSSubService[i] == 0x0F)  //clear the fota flash
									{
										if(UDSSubServiceActionCode[i] == 0x01)
										{
											//Clear the Flash
											boolRet = UDSClearFotaDownloadRegion();
											//if ok											
											UDSPositiveAnswer(0x05,i,(UINT32)boolRet);
										}
										else
										{
											UDSNegtiveAnswer(0x06,i,UDSSubServiceActionCode[i]<<8|0xFF);  //重点测试
										}										
									}
									else
									{
										UDSNegtiveAnswer(0x05,i,0xEE);  //the subservice is not surpported
									}
								}
								else
								{
									UDSNegtiveAnswer(0x05,i,0xFF);  //the survie is not surpported in current dialog mode
								}
								break;
							case 0x34:  //prepare for some one process
								if(UDSDialogMode == 3)
								{
									if(UDSSubService[i] == 0x0F)  //ask for download update data
									{									
										boolRet = UDSAskforDownLoadData();
										//if ok											
										UDSPositiveAnswer(0x05,i,(UINT32)boolRet);
									}
									else
									{
										UDSNegtiveAnswer(0x05,i,0xEE);
									}
								}
								else
								{
									UDSNegtiveAnswer(0x05,i,0xFF);
								}
								break;
							case 0x36:  //download the update data
								if(UDSDialogMode == 3)
								{
									if(downloadReady == TRUE)
									{
										counter++;  //记录报文数量									
										tempSliceCounter =  (CanRxMsg[i].Data[1]<<16)|(CanRxMsg[i].Data[2]<<8)|(CanRxMsg[i].Data[3]);
									/*
									#ifdef USING_PRINTF
										printf("tempSliceCounter = %x\n",tempSliceCounter);
									#endif
									*/										
										if(tempSliceCounter < (updateDifferDataPackageCounter+1)*25 && tempSliceCounter>=updateDifferDataPackageCounter*25)
										{
											updateDifferDataSliceCounter = tempSliceCounter % 25;
											for(j=0;j<4;j++)
											{
												updateDataBuffer[updateDifferDataSliceCounter*4+j] = CanRxMsg[i].Data[4+j];
											}								
											
											if(updateDifferDataSliceCounter>0)
											{
												sliceCounterFlag = sliceCounterFlag |(0x01<<updateDifferDataSliceCounter);
											}
											else
											{
												sliceCounterFlag = sliceCounterFlag |0x01;
											}
											/*
										#ifdef USING_PRINTF
											printf("sliceCounterFlag = %x  counter=%d\n",sliceCounterFlag,counter);
										#endif
										*/											
										}
										else
										{
											counter--;
										}
										
										if(counter == 25)
										{
											counter = 0;											
											if(sliceCounterFlag == 0x1FFFFFF)  //received all the 25 message of current package
											{															
												//write the buffer(100 byte) to flash
												ret = (uint8_t)BSP_QSPI_Write_Safe(updateDataBuffer,FLASH_FOTA_REGION_START+(updateDifferDataPackageCounter)*100,100);
												errorCount = 0;
												while(ret != QSPI_OK && errorCount<3)  //try to write most 3 times
												{
													errorCount++;
													BSP_QSPI_Erase_Safe(FLASH_FOTA_REGION_START+(updateDifferDataPackageCounter)*100,100);
													ret = (uint8_t)BSP_QSPI_Write_Safe(updateDataBuffer,FLASH_FOTA_REGION_START+(updateDifferDataPackageCounter)*100,100);
												}
												if(ret == QSPI_OK)  // write successed
												{
													UDSPositiveAnswer(0x08,i,tempSliceCounter);	
				/*									
													#if 0
													#ifdef USING_PRINTF
														UINT8 checksum0,checksum1=0;
														printf("updateDifferDataPackageCounter = %d\n",updateDifferDataPackageCounter);
														printf("updateDataBuffer write:\n");
														for(k=0;k<100;k++)
														{
															printf("%x  ",updateDataBuffer[k]);
															checksum0+=updateDataBuffer[k];
														}
														printf("\n");	
														
														UINT8 readFlash=0;
														printf("updateDataBuffer read:\n");
														
														for(k=0;k<100;k++)
														{
															BSP_QSPI_Read_Safe(&readFlash,FLASH_FOTA_REGION_START+updateDifferDataPackageCounter*100+k, 1);
															printf("%2x ",readFlash);
															checksum1+=readFlash;
														}
														printf("\n");
														if(checksum0 == checksum1)
														{
															printf("write succeed!\nchecksum0 = %2x,checksum1 = %2x\n",checksum0,checksum1);
														}
														else
														{
															printf("write fail!\nchecksum0 = %2x,checksum1 = %2x\n",checksum0,checksum1);
														}
														//printf("write succeed, next updateDifferDataPackageCounter = %x\n",updateDifferDataPackageCounter+1);
													#endif
													#endif
*/
													memset(updateDataBuffer,0, 100);
													sliceCounterFlag = 0x0;
													updateDifferDataPackageCounter++;
												}
												else  //write fail
												{
													UDSNegtiveAnswer(0x08,i,0xFFFFFFFF);  //failed to write this package to flash
													/*
													#if 0
													#ifdef USING_PRINTF
														printf("write flash fail!! updateDifferDataPackageCounter = %d\n",updateDifferDataPackageCounter);
														printf("ERROR: sliceCounterFlag=%x\n",sliceCounterFlag);
													#endif
													#endif
													*/
												}
											}
											else
											{
												//received 25 messages, but lose one or more messages
												//in this case, updater will try to send the package again(most 4 times)
												UDSNegtiveAnswer(0x08,i,sliceCounterFlag);
												#ifdef USING_PRINTF
														printf("ERROR: package lose one or mo message!! sliceCounterFlag = %x\n",sliceCounterFlag);
												#endif
												//sliceCounterFlag = 0x0;
												/*
											#ifdef USING_PRINTF
												printf("updateDifferDataPackageCounter = %d\n",updateDifferDataPackageCounter+1);
												printf("updateDataBuffer:\n");
												for(k=0;k<100;k++)
												{
													printf("%x  ",updateDataBuffer[k]);
												}
												printf("\n");
											#endif
											*/
												//memset(updateDataBuffer,0, 100);
											}
										}
									}
									else
									{
										UDSNegtiveAnswer(0x05,i,0xEF); //the download process is not ready
										counter = 0;
									}
								}
								else
								{
									UDSNegtiveAnswer(0x05,i,0xFF);  //the service is not surpported in current dialog mode
								}
								break;
							case 0x37:  //exit some one process
								if(UDSDialogMode == 3)
								{
									if(UDSSubService[i] == 0x0F)  // exit the download 
									 {
										UDSPositiveAnswer(0x04,i,0x00);
									/*
									#if 0
									#ifdef USING_PRINTF
										printf("CAN update data:\n");
										UINT8 ts=0;
										for(UINT32 jjj =0;jjj<updateDifferDataByteLen;jjj++)
										{											
											BSP_QSPI_Read_Safe(&ts,FLASH_FOTA_REGION_START+jjj,1);
											printf("%x ",ts);
										}
										printf("\n");
									#endif
									#endif
									*/
										downloadReady = FALSE;
									 }
									 else
									 {
										UDSNegtiveAnswer(0x05,i,0xEE);
									}
								}
								else
								{
									UDSNegtiveAnswer(0x05,i,0xFF);
								}
								break;
							default:								
								UDSNegtiveAnswer(0x04,i,0xFF);  //the service is not surpported
								break;
								
						}
					}
				}
			}
	//}
	for(i=0; i<2; i++)
	{
		UDSService[i] = 0;
		UDSSubService[i] = 0;
	}
}

UINT8 UDSPositiveAnswer(UINT8 answerLen,UINT8 messageIndex,UINT32 posCode)
{
	CAN_Msg_Type UDSAnswer;
	UINT8 ret;
	UDSAnswer.Id = 0x7A8;
	UDSAnswer.DLC = 8;
	UDSAnswer.Data[0] = answerLen;
	UDSAnswer.Data[1] = 0x78;
	UDSAnswer.Data[2] = UDSService[messageIndex];
	UDSAnswer.Data[3] = UDSSubService[messageIndex];
	if(answerLen == 4)
	{
		UDSAnswer.Data[4] = 0x00;
		UDSAnswer.Data[5] = 0x00;
		UDSAnswer.Data[6] = 0x00;
		UDSAnswer.Data[7] = 0x00;
	}
	else if(answerLen==5)
	{	UDSAnswer.Data[4] = posCode;
		UDSAnswer.Data[5] = 0x00;
		UDSAnswer.Data[6] = 0x00;
		UDSAnswer.Data[7] = 0x00;
	}
	else if(answerLen==6)
	{
		UDSAnswer.Data[4] = posCode>>8;
		UDSAnswer.Data[5] = posCode;
		UDSAnswer.Data[6] = 0x00;
		UDSAnswer.Data[7] = 0x00;
	}
	else if(answerLen==7)
	{
		UDSAnswer.Data[4] = posCode>>16;
		UDSAnswer.Data[5] = posCode>>8;
		UDSAnswer.Data[6] = posCode;
		UDSAnswer.Data[7] = 0x00;
	}
	else if(answerLen==8)
	{
		UDSAnswer.Data[4] = posCode>>24;
		UDSAnswer.Data[5] = posCode>>16;
		UDSAnswer.Data[6] = posCode>>8;
		UDSAnswer.Data[7] = posCode;
	}
	ret = HAL_Can_Transmit(UDSAnswer);
	return ret;
}

UINT8 UDSNegtiveAnswer(UINT8 answerLen,UINT8 messageIndex, UINT32 negCode)
{
	CAN_Msg_Type UDSAnswer;
	UINT8 ret;
	UDSAnswer.Id = 0x7A8;
	UDSAnswer.DLC = 8;
	UDSAnswer.Data[0] = answerLen;
	UDSAnswer.Data[1] = 0x7F;
	UDSAnswer.Data[2] = UDSService[messageIndex];
	UDSAnswer.Data[3] = UDSSubService[messageIndex];
	if(answerLen==4)
	{
		UDSAnswer.Data[3] = negCode;
		UDSAnswer.Data[4] = 0x00;
		UDSAnswer.Data[5] = 0x00;
		UDSAnswer.Data[6] = 0x00;
		UDSAnswer.Data[7] = 0x00;
	}
	if(answerLen==5)
	{	UDSAnswer.Data[4] = negCode;
		UDSAnswer.Data[5] = 0x00;
		UDSAnswer.Data[6] = 0x00;
		UDSAnswer.Data[7] = 0x00;
	}
	else if(answerLen==6)
	{
		UDSAnswer.Data[4] = negCode>>8;
		UDSAnswer.Data[5] = negCode;
		UDSAnswer.Data[6] = 0x00;
		UDSAnswer.Data[7] = 0x00;
	}
	else if(answerLen==7)
	{
		UDSAnswer.Data[4] = negCode>>16;
		UDSAnswer.Data[5] = negCode>>8;
		UDSAnswer.Data[6] = negCode;
		UDSAnswer.Data[7] = 0x00;
	}
	else if(answerLen==8)
	{
		UDSAnswer.Data[4] = negCode>>24;
		UDSAnswer.Data[5] = negCode>>16;
		UDSAnswer.Data[6] = negCode>>8;
		UDSAnswer.Data[7] = negCode;
	}
	ret = HAL_Can_Transmit(UDSAnswer);
	return ret;
}

BOOL UDSAskforDownLoadData()
{
	updateDifferDataPackageCounter = 0;
	downloadReady = true;
	
	return TRUE;
}

BOOL UDSClearFotaDownloadRegion()
{
	UINT8 ret = FALSE;
	ret = BSP_QSPI_Erase_Safe(FLASH_FOTA_REGION_START, 0x46000); //512k-32k -200k = 280k -> 0x46000
	
	if(ret == QSPI_OK)
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}
/*******
 * RT查询函数
 * 输入：R
 * 输出：T
 * 处理：利用R反查T
*******/
UINT8 LookUpRTtable(UINT32 R_value)
{
	UINT8 Temp_Table1[23] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22};
	UINT8 Temp_Table2[217] = {23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,
	49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,
	84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,
	115,116,117,118,119,120,121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,140,141,
	142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,160,161,162,163,164,165,166,167,168,
	169,170,171,172,173,174,175,176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191,192,193,194,195,
	196,197,198,199,200,201,202,203,204,205,206,207,208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,
	223,224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239};
	UINT32 R_Table1[23] = {202269,191063,180554,170694,161438,152746,144580,136905,129687,122898,116508,
						110493,104827,99488,94455,89710,85233,81008,77019,73252,69693,66329,63148};
	UINT16 R_Table2[217] = {
	60140,57293,54599,52049,49633,47343,45174,43117,41166,39315,37558,35891,34307,32802,31373,
	30014,28722,27493,26324,25211,24152,23144,22183,21268,20395,19564,18771,18015,17293,16604,
	15947,15319,14720,14147,13600,13077,12577,12098,11641,11203,10784,10383,10000,9632,9280,8942,
	8619,8309,8012,7727,7454,7191,6940,6698,6466,6244,6030,5825,5627,5438,5255,5080,4911,4749,4593,
	4443,4299,4160,4026,3898,3773,3654,3539,3428,3321,3218,3119,3023,2931,2841,2755,2672,2592,2515,
	2441,2369,2299,2232,2167,2104,2044,1985,1928,1874,1821,1769,1720,1672,1626,1581,1537,1495,1455,
	1415,1377,1340,1305,1270,1236,1204,1172,1142,1112,1083,1056,1029,1002,977,952,928,905,883,861,
	839,819,799,779,760,742,724,707,690,674,658,642,627,613,599,585,571,558,546,533,521,509,498,487,
	476,466,455,445,436,426,417,408,399,391,382,374,366,359,351,344,337,330,323,316,310,304,298,292,
	286,280,274,269,264,259,254,249,244,239,234,230,226,221,217,213,209,205,201,198,194,190,187,183,
	180,177,174,171,168,165,162,159,156,153,151,148,145,143,141,138,136,133,131,129,127,125};
	if(R_value>R_Table2[0])
	{
		if(R_value<=R_Table1[22])//判断是否超出表尾
		{
			return Temp_Table1[22];
		}
		else if(R_value>=R_Table1[0])//判断是否超出表头
		{
			return Temp_Table1[0];
		}
		else 
		{
			for(int i=0;i<23-1;i++)
			{
				if ((R_value<=R_Table1[i])&&(R_value>R_Table1[i+1]))//中间判断
				{
					return Temp_Table1[i];
				}
				else
				{
					continue;
				} 
				
			}
		}
	}//R-1
	else
	{
		if(R_value<=R_Table2[216])//判断是否超出表尾
		{
			return Temp_Table2[216];
		}
		else if(R_value>=R_Table2[0])//判断是否超出表头
		{
			return Temp_Table2[0];
		}
		else 
		{
			for(int i=0;i<217-1;i++)
			{
				if ((R_value<R_Table2[i])&&(R_value>=R_Table2[i+1]))//中间判断
				{
					return Temp_Table2[i+1];
				}
				else
				{
					continue;
				} 
				
			}
		}
	}//R-2
}

UINT16 GetErrorNum(UINT16 *ErrorArray,UINT8 Errorlen)
{
	UINT16 OutNum;
	OutNum = *(ErrorArray);
	for(int i=0;i<Errorlen-1;i++)
	{
		*(ErrorArray+i) = *(ErrorArray+i+1);
		if(*(ErrorArray+i+1)==0)
			break;
	}
	return OutNum;
}
UINT8 PutErrorNum(UINT16 *ErrorArray,UINT8 Errorlen,UINT16 ErrorNum)
{
	for(int i=0;i<Errorlen;i++)
	{
		if(*(ErrorArray+i)==0)
		{
			*(ErrorArray+i) = ErrorNum;
			return 0;
		}
		else
		{
			if(*(ErrorArray+i)==ErrorNum)
			{
				return 1;
			}
			else
			{
				continue;
			}
		}
	}
	return 2;
}